/**
 * FormValidation (https://formvalidation.io)
 * The best validation library for JavaScript
 * (c) 2013 - 2023 Nguyen Huu Phuoc <me@phuoc.ng>
 */

import { Plugin } from '@form-validation/core';

export interface RecaptchaOptions {
    // The ID of element showing the captcha
    element: string;
    // The language code defined by reCAPTCHA
    // See https://developers.google.com/recaptcha/docs/language
    language?: string;
    // The invalid message that will be shown in case the captcha is not valid
    // You don't need to define it if the back-end URL above returns the message
    message: string;
    // The site key provided by Google
    siteKey: string;
    backendVerificationUrl?: string;

    // The size of widget. It can be 'compact', 'normal' (default), or 'invisible'
    size?: string;

    // reCAPTCHA widget option (size can be 'compact' or 'normal')
    // See https://developers.google.com/recaptcha/docs/display

    // The theme name provided by Google. It can be light (default), dark
    theme?: string;

    // Invisible reCAPTCHA
    // See https://developers.google.com/recaptcha/docs/invisible
    // The position of reCAPTCHA. Can be 'bottomright' (default), 'bottomleft', 'inline'
    badge?: string;
}

export class Recaptcha extends Plugin<RecaptchaOptions> {
    // The captcha field name, generated by Google reCAPTCHA
    static CAPTCHA_FIELD: string;

    static DEFAULT_OPTIONS: RecaptchaOptions;

    // The name of callback that will be executed after reCaptcha script is loaded
    static LOADED_CALLBACK: string;

    constructor(opts?: RecaptchaOptions);
}
