/**
 * FormValidation (https://formvalidation.io)
 * The best validation library for JavaScript
 * (c) 2013 - 2023 Nguyen Huu Phuoc <me@phuoc.ng>
 */

import { Plugin } from '@form-validation/core';

export interface Recaptcha3Options {
    // The ID of element showing the captcha error
    element: string;
    // The language code defined by reCAPTCHA
    // See https://developers.google.com/recaptcha/docs/language
    language?: string;
    // Minimum score, between 0 and 1
    minimumScore?: number;
    // The invalid message that will be shown in case the captcha is not valid
    // You don't need to define it if the back-end URL above returns the message
    message: string;
    // The site key provided by Google
    siteKey: string;
    backendVerificationUrl: string;
    action: string;
}

export class Recaptcha3 extends Plugin<Recaptcha3Options> {
    // The captcha field name
    static CAPTCHA_FIELD: string;

    // The name of callback that will be executed after reCaptcha script is loaded
    static LOADED_CALLBACK: string;

    constructor(opts?: Recaptcha3Options);
}
