/**
 * FormValidation (https://formvalidation.io)
 * The best validation library for JavaScript
 * (c) 2013 - 2023 Nguyen Huu Phuoc <me@phuoc.ng>
 */

import { Plugin } from '@form-validation/core';

export interface WizardOptions {
    stepSelector: string;
    prevButton: string | HTMLElement;
    nextButton: string | HTMLElement;
    onStepActive?: (e: WizardStepEvent) => void;
    onStepInvalid?: (e: WizardStepEvent) => void;
    onStepValid?: (e: WizardStepEvent) => void;
    onValid?: (e: WizardValidEvent) => void;
    activeStepClass?: string;
    isFieldExcluded?: (field: string, element: HTMLElement, elements: HTMLElement[]) => boolean;
    isStepSkipped?: (event: WizardIsStepSkipped) => boolean;
    stepClass?: string;
}
export interface WizardStepEvent {
    step: number;
    numSteps: number;
}
export interface WizardValidEvent {
    numSteps: number;
}
export interface WizardIsStepSkipped {
    currentStep: number;
    numSteps: number;
    targetStep: number;
}

export class Wizard extends Plugin<WizardOptions> {
    static EXCLUDED_PLUGIN: string;
    constructor(opts?: WizardOptions);

    getCurrentStep(): number;
    goToPrevStep(): void;
    goToNextStep(): void;
    goToStep(index: number): void;
}
